<?php

/**
 * @file
 * Contains \Drupal\config_inspector\ConfigInspectorManager.
 */

namespace Drupal\config_inspector;

use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Config\TypedConfigManager;
use Drupal\Core\Config\Schema\Element;

/**
 * Manages plugins for configuration translation mappers.
 */
class ConfigInspectorManager extends DefaultPluginManager {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $configFactory;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $typedConfigManager;


  /**
   * Initialize new configuration inspector manager.
   *
   * @param ConfigFactory $config_factory
   * @param TypedConfigManager $typed_config_manager
   */
  public function __construct(ConfigFactory $config_factory, TypedConfigManager $typed_config_manager) {
    $this->configFactory = $config_factory;
    $this->typedConfigManager = $typed_config_manager;
  }

  /**
   * Provides definition of a configuration.
   *
   * @param string $plugin_id
   *   A string plugin ID.
   *
   * @return mixed|void
   */
  public function getDefinition($plugin_id) {
    return $this->typedConfigManager->getDefinition($plugin_id);
  }


  /**
   * Provides configuration data.
   *
   * @param string $name
   *   A string config key.
   *
   * @return array|null
   */
  public function getConfigData($name) {
    $old_state = $this->configFactory->getOverrideState();
    $this->configFactory->setOverrideState(FALSE);

    $config = $this->typedConfigManager->get($name)->getValue();

    $this->configFactory->setOverrideState($old_state);

    return $config;
  }

  /**
   * Provides configuration schema.
   *
   * @param string $name
   *   A string config key.
   *
   * @return array|null
   */
  public function getConfigSchema($name) {
    $old_state = $this->configFactory->getOverrideState();
    $this->configFactory->setOverrideState(FALSE);

    $config_schema = $this->typedConfigManager->get($name);

    $this->configFactory->setOverrideState($old_state);

    return $config_schema;
  }

  /**
   * Gets all contained typed data properties as plain array.
   *
   * @param array|object $schema
   *   An array of config elements with key.
   *
   * @return array
   *   List of Element objects indexed by full name (keys with dot notation).
   */
  public function convertConfigElementToList($schema) {
    $list = array();
    foreach ($schema as $key => $element) {
      if ($element instanceof Element) {
        foreach ($this->convertConfigElementToList($element) as $sub_key => $element) {
          $list[$key . '.' . $sub_key] = $element;
        }
      }
      else {
        $list[$key] = $element;
      }
    }
    return $list;
  }

}
