<?php

/**
 * @file
 * Contains \Drupal\config_inspector\Controller\ConfigInspectorController.
 */

namespace Drupal\config_inspector\Controller;

use Drupal\Component\Utility\String;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Config\StorageInterface;
use Drupal\config_inspector\configInspectorManager;
use Drupal\Core\Config\Schema\Element;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a controller for the config_inspector module.
 */
class ConfigInspectorController extends ControllerBase {

  /**
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $configStorage;

  /**
   * The configuration inspector manager.
   *
   * @var \Drupal\config_inspector\configInspectorManager
   */
  protected $configInspectorManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(StorageInterface $storage, configInspectorManager $config_inspector_manager) {
    $this->storage = $storage;
    $this->configInspectorManager = $config_inspector_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.storage'),
      $container->get('plugin.manager.config_inspector')
    );
  }

  /**
   * Builds a page listing all configuration keys to inspect.
   *
   * @return array
   *   A render array representing the list.
   */
  public function overview() {
    $list = array(
      '#type' => 'table',
      '#header' => array(
        'name' => t('Configuration key'),
        'list' => t('List'),
        'tree' => t('Tree'),
        'form' => t('Form'),
        'raw' => t('Raw data'),
      ),
    );

    foreach ($this->storage->listAll() as $name) {
      // Only handle elements with a schema. The schema system falls back on the
      // Property class for unknown types. See http://drupal.org/node/1905230
      $definition = $this->configInspectorManager->getDefinition($name);
      if (is_array($definition) && $definition['class'] == '\Drupal\Core\Config\Schema\Property') {
        continue;
      }

      $list[] = array(
        'name' => array('#markup' => $name),
        'list' => array('#markup' => l(t('List'), 'admin/reports/config-inspector/' . $name . '/list')),
        'tree' => array('#markup' => l(t('Tree'), 'admin/reports/config-inspector/' . $name . '/tree')),
        'form' => array('#markup' => l(t('Form'), 'admin/reports/config-inspector/' . $name . '/form')),
        'raw' => array('#markup' => l(t('Raw data'), 'admin/reports/config-inspector/' . $name . '/raw')),
      );
    }

    return $list;
  }


  /**
   * List (table) inspection view of the configuration
   *
   * @param $name
   *
   * @return array
   */
  public function getList($name) {
    $config_schema = $this->configInspectorManager->getConfigSchema($name);
    $output = $this->formatList($config_schema);
    $output['#title'] = $this->t('List of configuration data for %name', array('%name' => $name));
    return $output;
  }

  /**
   * Tree inspection view of the configuration.
   *
   * @param $name
   *
   * @return array
   */
  public function getTree($name) {
    $config_schema = $this->configInspectorManager->getConfigSchema($name);
    $output = $this->formatTree($config_schema);
    $output['#title'] = $this->t('Tree of configuration data for %name', array('%name' => $name));
    return $output;
  }

  /**
   * Form based configuration data inspection.
   */
  public function getForm($name) {
    $config_schema = $this->configInspectorManager->getConfigSchema($name);
    $output = \Drupal::formBuilder()->getForm('\Drupal\config_inspector\Form\ConfigInspectorItemForm', $config_schema);
    $output['#title'] = $this->t('Raw configuration data for %name', array('%name' => $name));
    return $output;
  }

  /**
   * Raw configuration data inspection.
   */
  public function getRawData($name) {
    $data = $this->configInspectorManager->getConfigData($name);
    $definition = $this->configInspectorManager->getDefinition($name);

    return array(
      '#title' => $this->t('Raw configuration data for %name', array('%name' => $name)),
      'config' => $this->formatData($data, 'Configuration data'),
      'schema' => $this->formatData($definition, 'Configuration schema'),
    );
  }

  /**
   * Format config schema as list table.
   */
  protected function formatList($schema) {
    $rows = array();
    $schema = $this->configInspectorManager->convertConfigElementToList($schema);
    foreach ($schema as $key => $element) {
      $definition = $element->getDataDefinition();
      $rows[] = array(
        $key,
        $definition['label'],
        $definition['type'],
        $element->getString(),
      );
    }
    return array(
      '#type' => 'table',
      '#header' => array(
        t('Name'),
        t('Label'),
        t('Type'),
        t('Value'),
      ),
      '#rows' => $rows,
    );
  }

  /**
   * Format config schema as a tree.
   */
  function formatTree($schema, $collapsed = FALSE, $base_key = '') {
    $build = array();
    foreach ($schema as $key => $element) {
      $definition = $element->getDataDefinition() + array('label' => t('N/A'));
      $type = $definition['type'];
      $element_key = $base_key . $key;
      if ($element instanceof Element) {
        $build[$key] = array(
            '#type' => 'details',
            '#title' => $definition['label'],
            '#description' => $element_key . ' (' . $type . ')',
            '#collapsible' => TRUE,
            '#collapsed' => $collapsed,
          ) + $this->formatTree($element, TRUE, $element_key . '.');
      }
      else {
        $value = $element->getString();
        $build[$key] = array(
          '#type' => 'item',
          '#title' => $definition['label'],
          '#markup' => String::checkPlain(empty($value) ? t('<empty>') : $value),
          '#description' => $element_key . ' (' . $type . ')',
        );
      }
    }
    return $build;
  }

  /**
   * Helper function to dump data in a reasonably reviewable fashion.
   */
  protected function formatData($data, $title = 'Data') {
    $output = '<h2>' . $title . '</h2>';
    $output .= '<pre>';
    $output .= htmlspecialchars(var_export($data, TRUE));
    $output .= '</pre>';
    $output .= '<br />';
    return array(
      '#markup' => $output
    );
  }

}
